# -*- coding:utf-8 -*-
###
# Created Date: Tuesday, October 10th 2017, 2:35:13 pm
# Author: Rossa <qingwen.liao@whu.edu.cn>
# -----
# Last Modified: Tue Oct 10 2017
# Modified By: Rossa
# -----
# Copyright (c) 2017 chancefocus
#
###
from nose.tools import assert_almost_equals

from paddy.models.entrust import Entrust


def test_Entrust():
    """Check if ``Entrust`` works
    """
    fake_value = {
        'stock_id': '000001', 'price': 100.0, 'volume': 34.2,
        'entrust_no': '183', 'status': '0'
    }
    entrust = Entrust(**fake_value)

    for key, value in fake_value.items():
        assert_almost_equals(getattr(entrust, key), value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: tests/test_account.py
# Author: Jimin Huang <huangjimin@whu.edu.cn>
# Date: 26.09.2017
from nose.tools import assert_almost_equals

from paddy.models.account import Account


def test_Account():
    """Check if ``Account`` works
    """
    fake_value = {
        'total_asset': 100.0, 'stock_asset': 100.0,
        'hold_stock': {'000001': 100.0}
    }
    account = Account(**fake_value)

    for key, value in fake_value.items():
        assert_almost_equals(getattr(account, key), value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: tests/unit/test_trader.py
# Author: Jimin Huang <huangjimin@whu.edu.cn>
# Date: 26.09.2017
import logging

from mock_logger import MockLoggingHandler
from nose.tools import assert_equals, assert_raises

from paddy import Trader
from paddy.constant import TRADE_TEST
from paddy.exceptions import ExecutedException
from paddy.mode.test_trader import Trader as TestTrader


HANDLER = MockLoggingHandler(level='DEBUG')


def test_Trader_init():
    """Check if ``Trader.__init__`` works
    """
    logger = logging.getLogger('Trader')
    logger.addHandler(HANDLER)
    trader = Trader(TRADE_TEST)

    assert isinstance(trader._trader, TestTrader)
    assert_equals(trader.mode, TRADE_TEST)
    assert_equals(
        HANDLER.messages['info'],
        [
            'Initialize trader as test_trader successful',
            'Initialize arguments: None'
        ]
    )


def test_Trader_not_allowed_methods():
    """Check if ``Trader`` raised Exception when methods not allowed
    """
    trader = Trader(TRADE_TEST)
    assert_raises(ExecutedException, getattr, trader, 'test')


def test_Trader_allowed_methods():
    """Check if ``Trader`` works when methods allowed
    """
    trader = Trader(TRADE_TEST)
    assert_equals(trader.get_account(), 'test')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: exceptions.py
# Author: Jimin Huang <huangjimin@whu.edu.cn>
# Date: 26.09.2017


class ExecutedException(Exception):
    """Exception class for execution
    """
    def __init__(self, error_info):
        self.error_info = error_info

    def __str__(self):
        return self.error_info


class CycleCancelFailedException(Exception):
    """Exception class for cycle cancel failed
    """
    def __init__(self, not_canceled_entrusts):
        self.not_canceled_entrusts = not_canceled_entrusts

    def __str__(self):
        return 'Canceled entrusts incomplete: {0}'.format(
            self.not_canceled_entrusts
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: trader.py
# Author: Jimin Huang <huangjimin@whu.edu.cn>
# Date: 26.09.2017
import importlib
import logging

from constant import TRADERS
from exceptions import ExecutedException


class Trader(object):
    """Abstract class for trading
    """
    __slots__ = [
        '_arguments', '_trader', '_logger', 'mode', 'account_id',
        'get_account', 'get_all_entrusts', 'get_cancellable_entrusts',
        'execute_entrust', 'cancel_all_entrusts', 'cycle_cancel_all_entrusts',
    ]

    def __init__(self, mode, arguments=None):
        """Initialize class with given mode and account_id

        Args:
            mode: an int to create correponding trading model
            arguments: an object for all arguments
        """
        self._logger = logging.getLogger(self.__class__.__name__)
        self._trader = importlib.import_module(
            '..mode.{0}'.format(TRADERS[mode]), __name__
        ).Trader(arguments)
        self.mode = mode
        self._logger.info(
            'Initialize trader as {0} successful'.format(TRADERS[mode])
        )
        self._logger.info('Initialize arguments: {0}'.format(arguments))

    def __getattr__(self, attr):
        """Modified the attribute visiting methods
        """
        self._logger.debug('Trader called {0}'.format(attr))
        if attr not in self.__slots__:
            raise ExecutedException('{0} not in allowed methods'.format(attr))
        return getattr(self._trader, attr)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: tests/test_trady_easy.py
# Author: Jimin Huang <huangjimin@whu.edu.cn>
# Date: 26.09.2017
import arrow
import json
import logging
import mock
import requests

from decimal import Decimal
from mock_logger import MockLoggingHandler
from nose.tools import assert_almost_equals, assert_equals, assert_raises
from requests.exceptions import HTTPError

from paddy.constant import (
    FAKE_ACCOUNT_RESPONSE, FAKE_ENTRUST_RESPONSE,
    FAKE_CANCELLABLE_ENTRUSTS_RESPONSE, SLEEP_TIME, MAX_REQUEST
)
from paddy.exceptions import ExecutedException, CycleCancelFailedException
from paddy.mode.trade_easy import (
    cycle_wait, Trader, response_to_account, LOGGER, response_to_entrusts
)
from paddy.models.entrust import Entrust


HANDLER = MockLoggingHandler(level='DEBUG')


def setup_module():
    LOGGER.addHandler(HANDLER)


class TestCycleWait(object):
    """Test class for ``trade_easy.cycle_wait``
    """
    def setUp(self):
        HANDLER.reset()

    def test_normal(self):
        """Check if ``trade_easy.cycle_wait`` works
        """
        @cycle_wait
        def normal(temp):
            pass

        start_time = arrow.now()
        normal('test')
        end_time = arrow.now()
        assert_equals((end_time - start_time).seconds, SLEEP_TIME)

    def test_raise_exception(self):
        """Check if ``trade_easy.cycle_wait`` works when exception raised
        """
        @cycle_wait
        def normal(temp):
            raise HTTPError()

        start_time = arrow.now()
        assert_raises(ExecutedException, normal, 'test')
        end_time = arrow.now()
        assert_equals(
            (end_time - start_time).seconds, SLEEP_TIME * MAX_REQUEST
        )
        assert_equals(HANDLER.messages['error'], ['', '', ''])


def test_response_to_account():
    """Check if ``trade_easy.response_to_account`` works
    """
    expect_results = {
        'total_asset': Decimal(367.94999999999999),
        'stock_asset': Decimal(245),
        'hold_stock': {
            '600022': Decimal(245.000), '000001': Decimal(1100.000)
        }
    }

    result = response_to_account(FAKE_ACCOUNT_RESPONSE)

    for key, value in expect_results.items():
        assert_almost_equals(getattr(result, key), value)


def test_response_to_entrusts():
    """Check if ``trade_easy.response_to_entrust`` works
    """
    expect_results = [
        {
            'stock_id': '600022', 'price': Decimal(2.540), 'volume': 100,
            'entrust_no': '10381', 'status': 1
        },
        {
            'stock_id': '600022', 'price': Decimal(0.000), 'volume': 0,
            'entrust_no': '10775', 'status': 3
        }
    ]

    results = response_to_entrusts(FAKE_ENTRUST_RESPONSE)
    for expect_result, result in zip(expect_results, results):
        for key, value in expect_result.items():
            assert_almost_equals(getattr(result, key), value)


class TestTrader(object):
    """Test class for ``trade_easy.Trader``
    """
    def setUp(self):
        logger = logging.getLogger(Trader.__name__)
        logger.addHandler(HANDLER)
        HANDLER.reset()
        self.trader = Trader({'host': '192.168.0.1:8080'})
        self.content = "response content"

    def test_init(self):
        """Check if ``trade_easy.Trader.__init__`` works
        """
        assert_equals(self.trader.host, '192.168.0.1:8080')
        assert_equals(HANDLER.messages['info'], ['Read trade easy arguments'])

    @mock.patch.object(requests, 'get')
    def test_get_account(self, mock_get):
        """Check if ``trade_easy.Trader.get_account`` works

        Args:
            mock_get: mock object of ``requests.get``
        """
        mock_get.return_value.json.return_value = FAKE_ACCOUNT_RESPONSE
        expect_results = {
            'total_asset': Decimal(367.94999999999999),
            'stock_asset': Decimal(245),
            'hold_stock': {
                '600022': Decimal(245.000), '000001': Decimal(1100.000)
            }
        }
        mock_get.return_value.content = self.content
        start_time = arrow.now()
        result = self.trader.get_account()
        end_time = arrow.now()

        assert_equals((end_time - start_time).seconds, SLEEP_TIME)
        assert_equals(
            HANDLER.messages['info'],
            [
                'Read trade easy arguments', 'response content',
                'Successfully get account info'
            ]
        )

        for key, value in expect_results.items():
            assert_almost_equals(getattr(result, key), value)

    @mock.patch.object(requests, 'get')
    def test_get_account_raise_exception(self, mock_get):
        """Check if ``trade_easy.Trader.get_account`` raises Exception

        Args:
            mock_get: mock object of ``requests.get``
        """
        mock_get.return_value.raise_for_status.side_effect = HTTPError()
        mock_get.return_value.content = self.content

        start_time = arrow.now()
        assert_raises(ExecutedException, self.trader.get_account)
        end_time = arrow.now()

        assert_equals(
            (end_time - start_time).seconds, SLEEP_TIME * MAX_REQUEST
        )
        assert_equals(
            HANDLER.messages['info'],
            [
                'Read trade easy arguments', 'response content',
                'response content', 'response content'
            ]
        )
        assert_equals(HANDLER.messages['error'], ['', '', ''])

    @mock.patch.object(requests, 'get')
    def test_get_all_entrusts(self, mock_get):
        """Check if ``trade_easy.Trader.get_all_entrusts

        Args:
            mock_get:mock object of ``requests.get``
        """
        mock_get.return_value.json.return_value = FAKE_ENTRUST_RESPONSE
        mock_get.return_value.content = self.content
        expect_results = [
            {
                'stock_id': '600022', 'price': Decimal(2.540), 'volume': 100,
                'entrust_no': '10381', 'status': 1
            },
            {
                'stock_id': '600022', 'price': Decimal(0.000), 'volume': 0,
                'entrust_no': '10775', 'status': 3
            }
        ]
        start_time = arrow.now()
        results = self.trader.get_all_entrusts()
        end_time = arrow.now()

        assert_equals((end_time - start_time).seconds, SLEEP_TIME)
        assert_equals(
            HANDLER.messages['info'],
            [
                'Read trade easy arguments', 'response content',
                'Successfully get entrust info'
            ]
        )
        assert_equals(len(results), len(expect_results))
        for each_result, each_expect_result in zip(results, expect_results):
            for key, value in each_expect_result.items():
                assert_almost_equals(getattr(each_result, key), value)

    @mock.patch.object(requests, 'get')
    def test_get_all_entrusts_raise_exception(self, mock_get):
        """Check if ``trade_easy.Trader.get_account`` raises Exception

        Args:
            mock_get: mock object of ``requests.get``
        """
        mock_get.return_value.raise_for_status.side_effect = HTTPError()
        mock_get.return_value.content = self.content
        start_time = arrow.now()
        assert_raises(ExecutedException, self.trader.get_all_entrusts)
        end_time = arrow.now()

        assert_equals(
            (end_time - start_time).seconds, SLEEP_TIME * MAX_REQUEST
        )
        assert_equals(
            HANDLER.messages['info'],
            [
                'Read trade easy arguments', 'response content',
                'response content', 'response content'
            ]
        )
        assert_equals(HANDLER.messages['error'], ['', '', ''])

    @mock.patch.object(requests, 'delete')
    def test_cancel_all_entrusts(self, mock_delete):
        """Check if ``trade_easy.Trader.cancel_all_entrusts`` works

        Args:
            mock_delete: mock object of ``requests.delete``
        """
        mock_delete.return_value.content = self.content
        start_time = arrow.now()
        result = self.trader.cancel_all_entrusts()
        end_time = arrow.now()

        mock_delete.assert_called_with('192.168.0.1:8080/orders')

        assert_equals((end_time - start_time).seconds, SLEEP_TIME)
        assert_equals(
            HANDLER.messages['info'],
            [
                'Read trade easy arguments', 'response content',
                'Successfully delete all orders'
            ]
        )
        assert_equals(result, True)

    @mock.patch.object(requests, 'delete')
    def test_cancel_all_entrusts_raises_exception(self, mock_delete):
        """Check if ``trade_easy.Trader.cancel_all_entrusts`` raises Exception

        Args:
            mock_delete: mock object of ``requests.delete``
        """
        mock_delete.return_value.raise_for_status.side_effect = HTTPError()
        mock_delete.return_value.content = self.content
        start_time = arrow.now()
        assert_raises(ExecutedException, self.trader.cancel_all_entrusts)
        end_time = arrow.now()

        mock_delete.assert_called_with('192.168.0.1:8080/orders')

        assert_equals(
            (end_time - start_time).seconds, SLEEP_TIME * MAX_REQUEST
        )
        assert_equals(
            HANDLER.messages['info'],
            [
                'Read trade easy arguments', 'response content',
                'response content', 'response content'
            ]
        )
        assert_equals(HANDLER.messages['error'], ['', '', ''])

    @mock.patch.object(requests, 'post')
    def test_execute_entrust(self, mock_post):
        """Check if ``trade_easy.Trader.execute_entrust`` works

        Args:
            mock_post: mock object of ``requests.post``
        """
        mock_post.return_value.json.return_value = {
            "id": "11106", "action": "BUY", "symbol": "test", "priceType": "0",
            "price": 0.01, "amountProportion": "", "amount": 100
        }
        mock_post.return_value.content = self.content
        start_time = arrow.now()
        result = self.trader.execute_entrust('BUY', 'test', 0.01, 100)
        end_time = arrow.now()

        mock_post.assert_called_with(
            '192.168.0.1:8080/orders',
            data=json.dumps(
                {
                    "action": "BUY", "symbol": "test", "type": "LIMIT",
                    "priceType": "0", "price": 0.01, "amount": 100
                }
            )
        )

        assert_equals((end_time - start_time).seconds, SLEEP_TIME)
        assert_equals(
            HANDLER.messages['info'],
            [
                'Read trade easy arguments', 'response content',
                (
                    'Successfully execute order:\n'
                    "{'symbol': 'test', 'amount': 100, 'action': 'BUY', "
                    "'type': 'LIMIT', 'price': 0.01, 'priceType': '0'}"
                )
            ]
        )
        assert_equals(result, "11106")

    @mock.patch.object(requests, 'post')
    def test_execute_entrust_raises_exception(self, mock_post):
        """Check if ``trade_easy.Trader.execute_entrust`` raises exception

        Args:
            mock_post: mock object of ``requests.post``
        """
        mock_post.return_value.raise_for_status.side_effect = HTTPError()
        mock_post.return_value.content = self.content
        start_time = arrow.now()
        assert_raises(
            ExecutedException, self.trader.execute_entrust, 'BUY', 'test',
            0.01, 100
        )
        end_time = arrow.now()

        mock_post.assert_called_with(
            '192.168.0.1:8080/orders',
            data=json.dumps(
                {
                    "action": "BUY", "symbol": "test", "type": "LIMIT",
                    "priceType": '0', "price": 0.01, "amount": 100
                }
            )
        )

        assert_equals(
            (end_time - start_time).seconds, SLEEP_TIME * MAX_REQUEST
        )
        assert_equals(
            HANDLER.messages['info'],
            [
                'Read trade easy arguments', 'response content',
                'response content', 'response content'
            ]
        )
        assert_equals(HANDLER.messages['error'], ['', '', ''])

    @mock.patch.object(requests, 'get')
    def test_get_cancellable_entrusts(self, mock_get):
        """Check if ``trade_easy.Trader.get_cancellable_entrusts`` works

        Args:
            mock_get: mock object of ``requests.get``
        """
        mock_get.return_value.json.return_value = (
            FAKE_CANCELLABLE_ENTRUSTS_RESPONSE
        )
        mock_get.return_value.content = self.content
        expect_results = [
            {
                'stock_id': '600022', 'price': Decimal(0.000), 'volume': 0,
                'entrust_no': '11106', 'status': 0
            },
            {
                'stock_id': '600022', 'price': Decimal(0.000), 'volume': 0,
                'entrust_no': '11852', 'status': 0
            }
        ]
        start_time = arrow.now()
        results = self.trader.get_cancellable_entrusts()
        end_time = arrow.now()

        assert_equals((end_time - start_time).seconds, SLEEP_TIME)
        assert_equals(
            HANDLER.messages['info'], [
                'Read trade easy arguments', 'response content',
                'Successfully get cancellable entrusts'
            ]
        )
        assert_equals(len(results), len(expect_results))
        for each_result, each_expect_result in zip(results, expect_results):
            for key, value in each_expect_result.items():
                assert_equals(getattr(each_result, key), value)

    @mock.patch.object(requests, 'get')
    def test_get_cancellable_entrusts_raise_exception(self, mock_get):
        """Check if ``trade_easy.Trader.get_cancellable_entrusts`` raises Exception

        Args:
            mock_get: mock object of ``requests.get``
        """
        mock_get.return_value.raise_for_status.side_effect = HTTPError()
        mock_get.return_value.content = self.content

        start_time = arrow.now()
        assert_raises(ExecutedException, self.trader.get_cancellable_entrusts)
        end_time = arrow.now()

        mock_get.assert_called_with('192.168.0.1:8080/orders?status=open')
        assert_equals(
            (end_time - start_time).seconds, SLEEP_TIME * MAX_REQUEST
        )
        assert_equals(
            HANDLER.messages['info'],
            [
                'Read trade easy arguments', 'response content',
                'response content', 'response content'
            ]
        )
        assert_equals(HANDLER.messages['error'], ['', '', ''])

    @mock.patch.object(Trader, 'cancel_all_entrusts')
    @mock.patch.object(Trader, 'get_cancellable_entrusts')
    def test_cycle_cancel_all_entrusts(
        self, mock_get_cancellable_entrusts, mock_cancel_all_entrusts
    ):
        """Check if ``trade_easy.Trader.cycle_cancel_all_entrusts`` works

        Args:
            mock_get_cancellable_entrusts: (
                mock object of ``Trader.get_cancellable_entrusts``
            )
            mock_cancel_all_entrusts: (
                mock object of ``Trader.cancel_all_entrusts``
            )
        """
        mock_cancel_all_entrusts.return_value = True
        mock_get_cancellable_entrusts.return_value = []

        start_time = arrow.now()
        result = self.trader.cycle_cancel_all_entrusts()
        end_time = arrow.now()

        assert_equals((end_time - start_time).seconds, 0)
        assert_equals(
            HANDLER.messages['info'], [
                'Read trade easy arguments',
                'Successfully cycle cancle all entrusts'
            ]
        )
        assert_equals(HANDLER.messages['error'], [])
        assert_equals(True, result)

    @mock.patch.object(Trader, 'cancel_all_entrusts')
    @mock.patch.object(Trader, 'get_cancellable_entrusts')
    def test_cycle_cancel_all_subscribe(
        self, mock_get_cancellable_entrusts, mock_cancel_all_entrusts
    ):
        """Check if ``trade_easy.Trader.cycle_cancel_all_entrusts`` subscribe

        Args:
            mock_get_cancellable_entrusts: (
                mock object of ``Trader.get_cancellable_entrusts``
            )
            mock_cancel_all_entrusts: (
                mock object of ``Trader.cancel_all_entrusts``
            )
        """
        mock_cancel_all_entrusts.return_value = True
        mock_get_cancellable_entrusts.return_value = [
            Entrust('000001', Decimal(0.00), 0, '10001', 5),
            Entrust('000001', Decimal(0.00), 0, '10002', 5)
        ]

        start_time = arrow.now()
        result = self.trader.cycle_cancel_all_entrusts()
        end_time = arrow.now()

        assert_equals((end_time - start_time).seconds, 0)
        assert_equals(
            HANDLER.messages['info'], [
                'Read trade easy arguments',
                'Successfully cycle cancle all entrusts'
            ]
        )
        assert_equals(HANDLER.messages['error'], [])
        assert_equals(True, result)

    @mock.patch.object(Trader, 'cancel_all_entrusts')
    @mock.patch.object(Trader, 'get_cancellable_entrusts')
    def test_cycle_cancel_all_entrusts_incomplete(
        self, mock_get_cancellable_entrusts, mock_cancel_all_entrusts
    ):
        """Check if ``trade_easy.Trader.cycle_cancel_all_entrusts`` incomplete

        Args:
            mock_get_cancellable_entrusts: (
                mock object of ``Trader.get_cancellable_entrusts``
            )
            mock_cancel_all_entrusts: (
                mock object of ``Trader.cancel_all_entrusts``
            )
        """
        mock_cancel_all_entrusts.return_value = True
        mock_get_cancellable_entrusts.return_value = [
            Entrust('000001', Decimal(0.00), 0, '10001', 0),
            Entrust('000001', Decimal(0.00), 0, '10002', 5)
        ]

        start_time = arrow.now()
        assert_raises(
            CycleCancelFailedException, self.trader.cycle_cancel_all_entrusts
        )
        end_time = arrow.now()

        assert_equals((end_time - start_time).seconds, 0)
        assert_equals(HANDLER.messages['info'], ['Read trade easy arguments'])
        assert_equals(HANDLER.messages['error'], ["['10001']"])

    @mock.patch.object(Trader, 'cancel_all_entrusts')
    def test_cycle_cancel_all_entrusts_failed(self, mock_cancel_all_entrusts):
        """Check if ``trade_easy.Trader.cycle_cancel_all_entrusts`` raises exception

        Args:
            mock_cancel_all_entrusts: (
                mock object of ``Trader.cancel_all_entrusts``
            )
        """
        mock_cancel_all_entrusts.side_effect = ExecutedException('test')

        start_time = arrow.now()
        assert_raises(
            CycleCancelFailedException, self.trader.cycle_cancel_all_entrusts
        )
        end_time = arrow.now()

        assert_equals((end_time - start_time).seconds, 0)
        assert_equals(HANDLER.messages['info'], ['Read trade easy arguments'])
        assert_equals(
            HANDLER.messages['error'], ['test', 'test', 'test', '[]']
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File: __init__.py
# Author: Jimin Huang <huangjimin@whu.edu.cn>
# Date: 26.09.2017
from exceptions import ExecutedException, CycleCancelFailedException
from trader import Trader


__all__ = ['Trader', 'ExecutedException', 'CycleCancelFailedException']
__version__ = '1.1.2.2a1'
